@@ -68,6 +68,7 @@ static HildonIMUI *client = NULL;
 static GtkWidget *window = NULL, *label = NULL;
 static GtkWidget* buttons[5];
 static gint candword_index = 2;
+static gboolean bquotedbl = TRUE, bquotright = TRUE, bparen = TRUE;
 
 static GType hildon_im_keyboard_decorator_type = 0;
 static GtkWidgetClass *parent_class = NULL;
@@ -616,11 +617,30 @@ gboolean fake_key_press(guint keyval, guint state, gchar **str, gboolean *ret)
         *ret = TRUE;
         break;
       case '\"':
-        val = "“";
+        val = bquotedbl?"“":"”";
+        bquotedbl = !bquotedbl;
         *ret = TRUE;
         break;
       case '\'':
-        val = "”";
+        val = bquotright?"‘":"’";
+        bquotright = !bquotright;
+        *ret = TRUE;
+        break;
+      case '+':
+        val = "·";
+        *ret = TRUE;
+        break;
+      case '-':
+        val = "——";
+        *ret = TRUE;
+        break;
+      case '_':
+        val = "……";
+        *ret = TRUE;
+        break;
+      case '&':
+        val = bparen?"《":"》";
+        bparen = !bparen;
         *ret = TRUE;
         break;
     }
